<?php
/**
 * This class handles all GeoDigs API requests
 */
namespace GeoDigs;

class API {
	const DEMO_AGENT_CODE = 'demo';
	const DEMO_KEY = '45b7d8043cf5ce94ccd3c844cd0a2f1f3d441ca4';
	const URL = 'http://api.geodigs.com/v2/';

	private static $key;
	private static $debug = 'disabled';
	private static $experimental = 'disabled';

	static function set_key( $key ) {
		self::$key = $key;
	}

	static function get_key() {
		return self::$key;
	}

	static function enable_debug() {
		self::$debug = 'enabled';
	}

	static function disable_debug() {
		self::$debug = 'disabled';
	}

	static function enable_experimental() {
		self::$experimental = 'enabled';
	}

	static function disable_experimental() {
		self::$experimental = 'disabled';
	}

	/**
	 * Sets up the call to the Geodigs API
	 * @param  string $method type of request
	 * @param  string $route  resource for request
	 * @param  array  $data   optional data to add onto the call
	 * @return json_object    API response
	 */
	static function call( $request, $route, $data = array() ) {
		$url = self::URL . $route;

		// Return an error if the request is now allowed
		if (
			$request != 'GET'
			&& $request != 'POST'
			&& $request != 'PUT'
			&& $request != 'DELETE'
		) {
			return array(
				'error' => true,
				'message' => 'Unknown GeoDigs request type ' . $request,
			);
		}

		// GET requests have a few alterations before going through to CURL
		if ( $request == 'GET' ) {
			// 'listingsPage' isn't a real param for the API.
			// It's used because WP uses 'page' already so if it exists
			// make sure to pass it as 'page' instead
			if ( isset( $data['listingsPage'] ) ) {
				$data['page'] = $data['listingsPage'];
				unset( $data['listingsPage'] );
			}

			// Add the data to the URL
			$url .= self::construct_parameters( $data );
			$data = array();
		}

		$response = self::curl_url( $url, $request, $data );
		return json_decode( $response );
	}

	/**
	 * Send CURL request
	 * @param  string $url       url to curl
	 * @param  array  $post_data post data to send, defaults to foo=bar because we must have data sent for the request to work
	 * @param  string $request   type of request
	 * @return obj               json object from response
	 */
	static function curl_url( $url, $request, $post_data = array( 'foo' => 'bar' ) ) {
		$ch = curl_init();
		curl_setopt( $ch, CURLOPT_URL, $url );
		curl_setopt( $ch, CURLOPT_RETURNTRANSFER, true );
		curl_setopt( $ch, CURLOPT_SSL_VERIFYPEER, false );
		curl_setopt( $ch, CURLOPT_HTTPHEADER, array(
			'API-Key: ' . self::$key,
			'API-Debug: ' . self::$debug,
			'API-Experimental: ' . self::$experimental,
		) );

		switch ( $request ) {
			case 'DELETE':
			case 'PUT':
	    		curl_setopt( $ch, CURLOPT_CUSTOMREQUEST, $request );
				break;

			case 'POST':
				curl_setopt( $ch, CURLOPT_POST, true );
				break;
		}

		if ( 'GET' != $request ) {
			curl_setopt( $ch, CURLOPT_POSTFIELDS, http_build_query( $post_data ) );
		}

		$response = curl_exec( $ch );

		if ( curl_errno( $ch ) ) {
			wp_die( '<h3>cURL Error</h3><br><strong>URL: </strong>' . $url . '<br><strong>Code: </strong>' . curl_errno( $ch ) . '<br><strong>Message: </strong>' . curl_error( $ch ),
		'cURL Error' );
		}

		// if (!$response) print_r(curl_getinfo($ch));

		curl_close( $ch );

		return $response;
	}

	/**
	 * Build a string to add to our url for an API call
	 * @param  array $params array of parameters to add
	 * @return string         URL encoded string of parameters
	 */
	private static function construct_parameters( $params = array() ) {
		if ( empty( $params ) == false ) {
			$params_str	= '?';
			$first		= true;

			foreach ( $params as $key => $value ) {
				// If this param is not the first one add the '&' symbol
				if ( !$first ) {
					$params_str .= '&';
				}
				else {
					$first = false;
				}

				// Add the key + value pair
				if ( is_array( $value ) || is_object( $value ) ) {
					$value = serialize( $value );
				}
				$params_str .= urlencode( $key ) . '=' . urlencode( $value );
			}

			return $params_str;
		}
		else {
			return '';
		}
	}
}
