<?php
/**
 * This class handles all GeoDigs page routing
 */
namespace GeoDigs;

class Router {
	static $listings_page;

	static function init() {
		if ( !is_admin() ) {
			add_action( 'pre_get_posts', array( __CLASS__, 'posts_fix' ) );
			// Removes post background.
			add_filter( 'post_class', array( __CLASS__, 'reset_classes' ) );
			// Keep the post from being edited.
			add_filter( 'get_edit_post_link', array( __CLASS__, 'disable_editing' ), 10, 3 );
			// Replace post content with our content.
			add_filter( 'the_posts', array( __CLASS__, 'display_page' ), PHP_INT_MAX );
			// Removes comments section.
			add_filter( "comments_template", array( __CLASS__, 'remove_comments' ), PHP_INT_MAX );

			// Remove's WP default styling
			remove_filter( 'the_content', 'wptexturize' );
			remove_filter( 'the_content', 'convert_smilies' );
			remove_filter( 'the_content', 'convert_chars' );
			remove_filter( 'the_content', 'wpautop' );
			remove_filter( 'the_content', 'prepend_attachment' );
		}
	}

	/**
	 * This ensures that other plugins, widgets, etc. that use the WP_Query object don't get our IDX data in their query.
	 * In "the_posts" filter we don't actually get the query, only the results, so we have to alter/check the data before hand
	 * and then restore the query in the filter.
	 * @param  object $query - WP_Query object reference
	 */
	static function posts_fix( $query ) {
		global $wp_query;

		if ( !is_array( $wp_query->query )
			|| !is_array( $query->query)
			|| isset( $wp_query->query['suppress_filters'] )
			|| isset( $query->query['suppress_filters'] )
		) {
			return;
		}

		if ( isset( $wp_query->query['gd_action'] ) ) {
			if ( !isset( $query->query['gd_action'] ) ) {
				$wp_query->query['gd_action_temp'] = $wp_query->query['gd_action'];
				unset( $wp_query->query['gd_action'] );
			} else {
				$query->query_vars['ignore_sticky_posts'] = true;
			}
		}
	}

	static function display_page( $posts ) {
		global $wp_query;

		// see posts_fix
		if ( is_array( $wp_query->query ) && isset( $wp_query->query['gd_action_temp'] ) ) {
			$wp_query->query['gd_action'] = $wp_query->query['gd_action_temp'];
			unset( $wp_query->query['gd_action_temp'] );
			return $posts;
		}

		// Figure out what kind of page to display
		if ( isset( $wp_query->query_vars['gd_action'] ) ) {
			// Removes link from title and comment section
			$wp_query->found_posts = 0;
			$wp_query->max_num_pages = 0;
			$wp_query->is_page= 1;
			$wp_query->is_home = null;
			$wp_query->is_singular = 1;

			// Get first post and save its details
			$page_data = '';
			$post_id = time();

			// By default always show the footer (disclaimers)
			$show_footer = true;

			switch ( urldecode( $wp_query->query_vars['gd_action'] ) ) {
				case 'account-home':
					User::require_login( $_SERVER['REQUEST_URI'] );
					$show_footer = false;

					ob_start();
					include GD_DIR_INCLUDES . 'account-home.php';

					$page_data = array(
						'content' => ob_get_contents(),
						'description' => 'User home page',
						'title' => 'My Account',
					);
					ob_end_clean();
					break;

				case 'account-settings':
					User::require_login( $_SERVER['REQUEST_URI'] );
					$show_footer = false;
					$page_data   = self::display_account_settings_page();
					break;

				case 'add-favorite':
					User::require_login( $_SERVER['REQUEST_URI'] );
					User::add_favorite( $_GET['listing_id'] );
					break;

				case 'advanced-search':
					$show_footer = false;
					$page_data   = array(
						'content' => '[gd_advanced_search]',
						'description' => 'Real estate advanced search tool',
						'title' => 'Advanced Search',
					);
					break;

				case 'delete-favorite':
					User::require_login( $_SERVER['REQUEST_URI'] );
					User::delete_favorite( $_GET['listing_id'] );
					break;

				case 'details':
					if ( isset( $wp_query->query_vars['gd_listing_id'] ) ) {
						$listing_id = $wp_query->query_vars['gd_listing_id'];

						if ( isset( $_SESSION['gd']['user'] ) || gd_under_detail_view_limit( $listing_id ) == true ) {
							$listing = API::call( 'GET', 'listings/' . $listing_id );
                            $first_image = gd_create_listing_image_url( $listing, 'large' );

							ob_start();

                            // This is for SEO plugins that choose the OG image
                            // based on the first image found.
                            echo '<img src="' . $first_image . '" style="display:none;" />';

							// Gets the HTML for our results.
							include GD_DIR_INCLUDES . 'listing-detail.php';

							$page_data = array(
								'content'     => ob_get_clean(),
								'description' => $listing->description,
								'title'       => $listing->address->readable,
                                'name'        => $_SERVER['REQUEST_URI'],
							);
						} else {
							$_SESSION['gd_redirect_url'] = $_SERVER['REQUEST_URI'];

							ob_start();
							include GD_DIR_INCLUDES . 'detail-view-count-reached.php';
							$page_data = array(
								'content'     => ob_get_clean(),
								'description' => 'Require login to view more',
								'title'       => 'View Limit Reached',
                                'name'        => $_SERVER['REQUEST_URI'],
							);
						}
					}
					break;

				case 'download-document':
					global $wpdb;

					$id = $wp_query->query_vars['doc_id'];
					DocumentStore::download_file($id);
					exit;

				case 'favorites':
					User::require_login( $_SERVER['REQUEST_URI'] );
					$page_data = self::display_favorites_page();
					break;

				case 'forgot-password':
					// Logs out the user if they are logged in but doesn't redirect them to the home page
					User::log_out( false );
					$show_footer = false;
					$page_data   = self::display_forgot_password_page();
					break;

				case 'home-worth':
					$show_footer = false;

					ob_start();
					include GD_DIR_INCLUDES . 'home-worth-form.php';
					$page_data = array(
						'content'		=> ob_get_contents(),
						'description'	=> 'Geodigs What\'s My Home Worth Calculator',
						'title'			=> 'What\'s My Home Worth',
					);

					ob_end_clean();
					break;

				case 'listing-alerts':
					User::require_login( $_SERVER['REQUEST_URI'] );

					ob_start();

					include GD_DIR_INCLUDES . 'listing-alerts.php';

					$page_data = array(
						'content'		=> ob_get_contents(),
						'description'	=> 'Geodigs Listing Alerts Page',
						'title'			=> $page_title,
					);

					ob_end_clean();
					break;

				case 'login':
					$show_footer = false;

					// Logs out the user if they are logged in but doesn't redirect them to the home page
					User::log_out( false );
					$page_data = self::display_login_page();
					break;

				case 'log-out':
					User::log_out();
					break;

				case 'more-info':
					//User::require_login( $_SERVER['REQUEST_URI'] );

					ob_start();

					include GD_DIR_INCLUDES . 'more-info-form.php';

					$show_footer = false;
					$page_data   = array(
						'content'     => ob_get_contents(),
						'description' => 'Get more info for a listing from the realtor',
						'title'       => 'More Information',
					);

					ob_end_clean();
					break;

				case 'more-info-requested':
					//User::require_login( $_SERVER['REQUEST_URI'] );

					ob_start();

					include GD_DIR_INCLUDES . 'more-info-requested.php';

					$show_footer = false;
					$page_data   = array(
						'content'     => ob_get_contents(),
						'description' => 'Show success message for more info request',
						'title'       => 'Request Sent',
					);

					ob_end_clean();
					break;

				case 'our-listings':
					$page_data = array(
						'content' => '[gd_our_listings]',
						'description' => 'Our listings',
						'title' => 'Our Listings',
					);
					break;

				case 'proxy-api':
					header( 'Content-Type: application/json' );

					switch ( urldecode( $wp_query->query_vars['api_action'] ) ) {
						case 'get-statuses':
							echo json_encode( API::call( "GET", "listings/statuses" ) );
							exit;

						case 'get-styles':
							echo json_encode( API::call( "GET", "listings/styles" ) );
							exit;

						case 'get-types':
							echo json_encode( API::call( "GET", "listings/types" ) );
							exit;

						case 'get-sort':
							echo json_encode( API::call( "GET", "listings/sort" ) );
							exit;
					}
					break;

				case 'search':
					$general_options = gd_get_option( GD_OPTIONS_GENERAL );
					$data = $_GET;
					$data['limit'] = $general_options['ListingsPerPage'];

					$results = API::call( 'GET', 'listings', $data );
					$page_data = self::display_search_results( $results );
					break;

				case 'signup':
					// Logs out the user if they are logged in but doesn't redirect them to the home page
					User::log_out( false );
					$show_footer = false;

					if (
						isset( $_POST['firstName'] ) &&
						isset( $_POST['lastName'] ) &&
						isset( $_POST['phone'] ) &&
						isset( $_POST['email'] ) &&
						isset( $_POST['password'] )
					) {
						$errors    = gd_process_signup();
						$page_data = count( $errors ) > 0 ? self::display_signup_page( $errors ) : self::display_signup_success_page();
					} else {
						$page_data = self::display_signup_page();
					}
					break;

				default:
					// Do nothing.
					break;
			}

			// Setup the post
			$post = (object) array(
				"ID"             => $post_id,
				"post_content"   => $page_data['content'],
				"post_name"      => isset( $page_data['name'] ) ? $page_data['name'] : 'geodigs',
				"post_title"     => $page_data['title'],
				"post_status"    => "publish",
				"post_author"    => 1,
				"ping_status"    => "closed",
				"post_parent"    => 0,
				"post_type"      => 'page',
				"post_excerpt"   => $page_data['description'],
				"post_date"      => date( "c" ),
				"post_date_gmt"  => gmdate( "c" ),
				"comment_count"  => 0,
				"comment_status" => "closed",
			);

			wp_cache_set( $post_id, $post, 'posts' );

			// By not cleaning out the posts array multiple posts are showing on pages
			// so it is commented out until we found out why we did this in the first place
			// array_unshift( $posts, $post );
			$posts = array( $post );
		}

		return $posts;
	}

	private static function display_favorites_page() {
		$results          = API::call( 'GET', 'favorites' );
		$show_edit_search = false;
		$show_pagination  = false; // "Favorites" doesn't have pages
		$show_count		  = false; // "Favorites" doesn't have pages

		// Begin output of page
		ob_start();

		include_once GD_DIR_INCLUDES . 'listings-results.php';

		$page_data = array(
			'content'     => ob_get_contents(),
			'description' => 'Geodigs User Favorites Page',
			'title'       => 'Favorites',
		);

		ob_end_clean();
		return $page_data;
	}

	private static function display_account_settings_page() {
		ob_start();
		include_once GD_DIR_INCLUDES . 'account-settings.php';
		$content = ob_get_contents();
		ob_end_clean();

		$page_data = array(
			'content'		=> $content,
			'description'	=> 'Geodigs User Account Settings Page',
			'title'			=> 'Account Settings',
		);

		return $page_data;
	}

	private static function display_forgot_password_page() {
	    global $login_info;

		$errors = array();

		// Send API call and figure out what page to display
		if ( $_POST['email'] ) {
            $forgot_pw_request = API::call( 'GET', 'forgot-password', array(
                'email'     => $_POST['email'],
                'agentCode' => $login_info['AgentCode']
            ));

			if ( isset( $forgot_pw_request->error ) ) {
				$page_type = 'form';
				$errors['not_found'] = $forgot_pw_request->message;
			}
			else {
				$page_type = 'success';
			}
		}
		else {
			$page_type = 'form';
		}

		// Output HTML
		ob_start();

		include_once GD_DIR_INCLUDES . 'forgot-password.php';

		$page_data = array(
			'content'		=> ob_get_contents(),
			'description'	=> 'Geodigs User Forgot Password Page',
			'title'			=> 'Forgot Password',
		);

		ob_end_clean();
		return $page_data;
	}

	private static function display_signup_page( $errors = array() ) {
		$title       = 'Sign Up';
		$description = 'GeoDigs User Sign Up Page';
		$content     = Template::load(
			'account/register.php',
			array(
				'errors' => $errors,
			),
			false
		);

		$page_data = array(
			'content'     => $content,
			'description' => $description,
			'title'       => $title,
		);

		return $page_data;
	}

	private static function display_signup_success_page() {
		$title       = 'Success';
		$description = 'GeoDigs User Sign Up Success Page';
		$content     = Template::load(
			'account/signup-success.php',
			array(),
			false
		);

		$page_data = array(
			'content'     => $content,
			'description' => $description,
			'title'       => $title,
		);

		return $page_data;
	}

	private static function display_login_page() {
		if ( isset( $_POST['email'] ) && isset( $_POST['password'] ) ) {
			// Encrypt password
			$_POST['password'] = sha1( $_POST['password'] . 'geodigs' );

			$login_request = API::call( 'POST', 'users/login', $_POST );
			if ( isset( $login_request->error ) ) {
				// Return to login form with the error
				return self::create_login_form( $login_request->message );
			} else {
				User::login( $login_request );
			}
		} else {
			return self::create_login_form();
		}
	}

	private static function create_login_form( $error = null ) {
		$content = Template::load(
			'account/login.php',
			array(
				'error' => $error,
			),
			false
		);

		$page_data = array(
			'content' => $content,
			'description' => 'Geodigs User Login Page',
			'title' => 'Login',
		);

		return $page_data;
	}

	private static function display_search_results( $results ) {
		// Save our search link
		$_POST['results_url'] = $_SERVER['REQUEST_URI'];

		// Begin output of page
		ob_start();

		include_once GD_DIR_INCLUDES . 'listings-results.php';

		$page_data = array(
			'content' => ob_get_contents(),
			'description' => 'Real estate search results',
			'title' => 'Search Results',
		);

		ob_end_clean();
		return $page_data;
	}

	static function set_redirect_uri( $uri ) {
		$_SESSION['gd_redirect_url'] = PROTOCOL . $_SERVER["HTTP_HOST"] . $uri;
	}

	static function reset_classes( $classes ) {
		global $wp_query;

		if ( isset( $wp_query->query['gd_action'] ) ) {
			$classes = array_diff( $classes, array( 'hentry' ) );
		}
		return $classes;
	}

	static function remove_comments( $path ){
		global $wp_query;

		if ( isset( $wp_query->query['gd_action'] ) ) {
			return GD_DIR_INCLUDES . 'comments.php';
		}
		return $path;
	}

	static function disable_editing( $edit_link, $post_id, $context ) {
		return;
	}
}
Router::init();
