<?php
namespace GeoDigs;

/**
 * Handles GeoDigs users aka the agents' clients
 */
class User {

	/**
	 * Login using the API data
	 * @param $api_data array Data retrieved from the API
	 */
	static function login( $api_data ) {
		// Remove any previous user data
		self::log_out( false );

		// Store user information
    self::store_cookie( $api_data->apiKey, $api_data->userId );
		self::store_user_info( $api_data->userId );

		// Get our favorite listing ids
		self::get_favorites();

		$redirect_url = home_url();
		// If a redirect URL was supplied use it and unset it after
		if ( isset( $_SESSION['gd_redirect_url'] ) ) {
			$redirect_url = $_SESSION['gd_redirect_url'];
			unset( $_SESSION['gd_redirect_url'] );
		}

		// Reset view count history
		gd_reset_detail_view_count();

		// Redirect
		wp_redirect( $redirect_url );
		exit;
	}

	/**
	 * Log out user
	 * @param  boolean $redirect Redirect to home page
	 * @param  boolean $force    Force log out
	 */
	static function log_out( $redirect = true, $force = false ) {
		if ( self::is_logged_in() || $force ) {
			// Delete user session
			unset( $_SESSION['gd']['user'] );
			unset( $_COOKIE['geodigs'] );
			unset( $_COOKIE['geodigs_temp'] );
			setcookie( 'geodigs', '', time() - 3600, '/' );
			setcookie( 'geodigs_temp', '', time() - 3600, '/' );
		}

		if ( $redirect == true ) {
			// Redirect to home
			wp_redirect( home_url() );
			exit;
		}
	}

	/**
	 * Login the agent as an user
	 */
	static function login_as( $user_id ) {
		// Remove any previous user data
		self::log_out( false );

		// Get info for user
		self::store_user_info( $user_id );
		self::get_favorites();

		// Only login as another user for a day
		// When getting user info as an agent their API key is also returned so we can use it here
    self::store_cookie( $_SESSION['gd']['user']->apiKey, $user_id );
		API::set_key( $_SESSION['gd']['user']->apiKey );

		// Reset view count history
		gd_reset_detail_view_count();

		// Send agent to user home page
		header( 'Location: ' . home_url() . '/' . GD_URL_ACCOUNT );
		exit;
	}

	static function login_temp( $agentCode ) {
		global $login_info;

		// If we are in demo mode just use the demo key,
		// otherwise generate one
		if ( $agentCode == 'demo' ) {
			$key = $login_info['APIKey'];
		} else {
			$data = array(
				'agentCode' => $agentCode,
				'type' => 'web',
			);
			$response = API::call( 'POST', 'users', $data );
			$key = $response->apiKey;
			$user_id = $response->id;

      self::store_cookie( $key, $user_id, 'temp' );
		}

		API::set_key( $key );
	}

  static function store_cookie( $key, $id, $temp=false ) {
      $name = 'geodigs' . ( $temp ? '_temp' : '' );
			// Set cookie for 10 years
      $cookie = serialize( array( 'k' => $key, 'i' => $id ) );
			setcookie( $name, $cookie, time() + ( 10 * 365 * 24 * 60 * 60 ), '/' );
  }

	static function get_user_info( $userId ) {
		return API::call( 'GET', 'users/' . $userId );
	}

	static function store_user_info( $userId ) {
		$_SESSION['gd']['user'] = self::get_user_info( $userId );
    if ( false == isset( $_SESSION['gd']['user']->id ) ) {
      self::log_out( false, true ); // Don't redirect, force log out
      $link = '<a href="' . home_url( GD_URL_LOGIN, 'relative' ) . '">here</a>';
      wp_die("Could not get user information.  Click $link to login again." );
    }
	}

	static function update_user_info( $data ) {
		if ( $_SESSION['gd']['user'] = ( object ) array_merge(( array ) $_SESSION['gd']['user'], ( array ) $data ) ) {
			return true;
		} else {
			return false;
		}
	}

	/**
	 * Checks to see if there is a geodigs cookie
	 * @return bool User has geodigs cookie
	 */
	static function is_logged_in() {
		return isset( $_COOKIE['geodigs'] );
	}

	/**
	 * Requires the user to be logged in before proceeding
	 * @param	string	$redirect_url	url to redirect to after login is successful
	 */
	static function require_login( $redirect_url ) {
		if ( false == self::is_logged_in() ) {
			// Redirect to the home page if a url is not specified
			if ( false == isset( $redirect_url ) ) {
				$redirect_url = home_url();
			}
			Router::set_redirect_uri( $redirect_url );
			wp_redirect( home_url() . '/' . GD_URL_LOGIN );
			exit;
		}

    if (false == isset( $_SESSION['gd']['user']->id ) ) {
      $user_id = User::get_user_id();
      self::store_user_info( $user_id );
    }
	}

	/**
	 * Returns the user's API key for logged in users and temporary users
	 * @return string or bool - API key or false on failure
	 */
	static function get_api_key() {
		if ( isset( $_COOKIE['geodigs'] ) ) {
      $geodigs_cookie = unserialize($_COOKIE['geodigs']);
			return $geodigs_cookie['k'];
		} elseif ( isset( $_COOKIE['geodigs_temp'] ) ) {
      $geodigs_cookie = unserialize($_COOKIE['geodigs_temp']);
			return $geodigs_cookie['k'];
		} else {
			return false;
		}
	}

  /**
   * Returns the stored user's ID.  This is used to get the
   * rest of the information about the user.
   * @return int or bool - API user ID.
   */
  static function get_user_id() {
		if ( isset( $_COOKIE['geodigs'] ) ) {
      $geodigs_cookie = unserialize( $_COOKIE['geodigs'] );
			return $geodigs_cookie['i'];
		} elseif ( isset( $_COOKIE['geodigs_temp'] ) ) {
      $geodigs_cookie = unserialize( $_COOKIE['geodigs_temp'] );
			return $geodigs_cookie['i'];
		} else {
			return false;
    }
  }

	/**
	 * Checks to see if there is a user object stored in the session data
	 * @param  bool $redirect	redirect to login page if not logged in
	 * @return bool				user object is set
	 */
	static function has_favorite( $listing_id ) {
		if ( isset( $_SESSION['gd']['user']->favorites->listings ) ) {
			foreach ( $_SESSION['gd']['user']->favorites->listings as $listing ) {
				if ( $listing->id == $listing_id ) {
					return true;
				}
			}
		}

		return false;
	}

	/**
	 * Gets the user's favorite listings id's
	 */
	static function get_favorites() {
		$response = API::call( 'GET', 'favorites', array( 'fields' => 'id' ) );
		if ( !isset( $response->error ) ) {
			$_SESSION['gd']['user']->favorites = $response;
		}
	}

	/**
	 * Add a favorite
	 * @param $listing_id string ID of listing to add
	 */
	static function add_favorite( $listing_id ) {
		$response = API::call( 'POST', 'favorites/' . $listing_id );
		// Get our new favorite listing ids
		self::get_favorites();

		wp_send_json( $response );
	}

	/**
	 * Delete a favorite
	 * @param $listing_id string ID of listing to delete
	 */
	static function delete_favorite( $listing_id ) {
		$response = API::call( 'DELETE', 'favorites/' . $listing_id );
		// Get our new favorite listing ids
		self::get_favorites();

		wp_send_json( $response );
	}

	/**
	 * Gets the user's saved searches
	 * @param string $format Format to store in
	 */
	static function get_searches( $format = 'object' ) {
		$response = API::call( 'GET', 'searches' );
		$is_obj = is_object( $response );
		if ( is_array( $response ) || $is_obj && false == isset( $response->error ) ) {
			if ( $format == 'array' && $is_obj ) {
				$_SESSION['gd']['user']->searches = get_object_vars( $response );
			}
			else {
				$_SESSION['gd']['user']->searches = $response;
			}
		}
	}

	/**
	 * Get a saved search
	 * @param	$search_id string	ID of search to get
	 * @return	$array				Search details
	 */
	static function get_search( $search_id ) {
		$response   = API::call( 'GET', 'searches/' . $search_id ); // Returns as an object with each result being an object of that object
		$search     = get_object_vars( $response ); // Convert the request to an array

		return $search[$search_id]; // Get the first item in the array which is our chosen search
	}

	/**
	 * Save a search for the user to use for listing alerts
	 */
	static function add_search() {
		// Convert our params to a string
		$param_str = '';
		foreach ( $_POST as $key => $value ) {
			if ( $key != 'push' && $key != 'text' && $key != 'email' && $key != 'freq' && $key != 'search_name' ) {
				if ( $value != '' ) {
					if ( $key == 'polygon' ) {
						// Replace the commas because the API recognizes commas as metacharacters
						$value = str_replace( ',', '|', $value );
					}
					$param_str .= $key . ':' . $value . ',';
				}
				unset( $_POST[$key] );
			}
		}
		$_POST['params'] = rtrim( $param_str, ',' );

		// Make sure we fill all of our API required vars
		if ( !isset( $_POST['push'] ) ) {
			$_POST['push'] = '0';
		}
		if ( !isset( $_POST['text'] ) ) {
			$_POST['text'] = '0';
		}
		if ( !isset( $_POST['email'] ) ) {
			$_POST['email'] = '0';
		}

		// WP uses 'name' so we can only use it temorarily
		if ( isset( $_POST['name'] ) ) {
			$name = $_POST['name'];
		}
		$_POST['name'] = $_POST['search_name'];
		unset( $_POST['search_name'] );

		// Make API request
		$response = API::call( 'POST', 'searches', $_POST );

		// Reset 'name'
		if ( isset( $name ) ) {
			$_POST['name'] = $name;
		}

		return $response;
	}

	/**
	 * Delete a saved search
	 * @param $search_id string	ID of search to delete
	 */
	static function delete_search( $search_id ) {
		$response = API::call( 'DELETE', 'searches/' . $search_id );
		return isset( $response->error );
	}

	static function notify( $email, $subject, $message ) {
		$from = get_bloginfo( 'admin_email' );
		$headers = 'From: ' . $from . "\r\n" .
    'Reply-To: ' . $from . "\r\n" .
    'X-Mailer: PHP/' . phpversion();

		mail( $email, $subject, $message );
	}
}
