<?php
use GeoDigs\API as API;

/**
 * Returns a GeoDigs setting and it's defaults
 *
 * @param  string $option GeoDigs setting.
 * @return array         GeoDigs settings
 */
function gd_get_option( $option ) {
    $defaults = array();

    switch ( $option ) {
        case GD_OPTIONS_LOGIN:
            $defaults = array(
                'APIKey' => API::DEMO_KEY,
                'AgentCode' => 'demo',
                'Status' => 'demo',
            );
            break;

        case GD_OPTIONS_ADVANCED_SEARCH:
            $defaults = array(
                'MapSearchStartLat' => 39.7392, // Defaults to Denver.
                'MapSearchStartLon' => -104.9903, // Defaults to Denver.
                'MapSearchStartZoom' => 12,
                'UseCustomCities' => false,
            );
            break;

        case GD_OPTIONS_GENERAL:
            $defaults = array(
                'HideLoginLinks' => false,
                'ListingsPerPage' => 10,
                'MaxDetailViews' => 5,
                'MoreInfoEmail' => 'agent',
            );

            break;

        case GD_OPTIONS_OUR_LISTINGS:
            // Get the agent's first source available
            // Might be a better way to do this but for now this will do.
            $primary_source = null;
            for ( $i = 0; $i < 10; $i++ ) {
                if ( isset( $_SESSION['gd']['agent']->sources->{$i} ) ) {
                    $primary_source = $i;
                }
            }
            $defaults = array(
                'ListingsToDisplay' => 'agent',
                'PrimarySource' => $primary_source,
                'Type' => 'all',
            );
            break;

        case GD_OPTIONS_SCRIPTS_AND_STYLES:
            $defaults = array(
                'DequeuedScripts' => array(),
                'DequeuedStyles' => array(),
            );
            break;

        case GD_OPTIONS_GOOGLE_MAPS:
            $defaults = array(
                'ApiKey' => 'AIzaSyChMx8AYfbFuDIzmbABuZYwB_4ivGW1a8s',
            );
            break;

        case GD_OPTIONS_URLS:
            $defaults = array(
                'ListingsPrefix' => 'real-estate/',
                'AccountPrefix' => 'account/',
                'FavoritesPrefix' => 'favorites/',
                'ProxyApi' => 'geodigs/api/',
                'DocumentStore' => 'document-store',
                'HomeWorth' => 'home-worth',
                'ListingAlerts' => 'listing-alerts',
            );
            break;
    }

    // This ensures that if we don't have every option the defaults fill them in.
    $options = get_option( $option, $defaults );
    if ( ! is_array( $options ) ) {
        $options = array();
    }
    return array_merge( $defaults, $options );
}

// function gd_add_wp_params( $params ) {
//  global $wp_params;
//  $wp_params = array_merge( $wp_params, $params );
// }

/**
 * Get agent login information.
 *
 * @param  string $param Optional specific parameter to get.
 * @return varies        Agent info.
 */
function gd_get_agent_login_info( $param = '' ) {
    global $login_info;

    // If this function is called without a param return all.
    if ( '' === $param ) {
        return $login_info;
    } else {
        if ( isset( $login_info[ $param ] ) ) {
            return $login_info[ $param ];
        } else {
            return false;
        }
    }
}

/**
 * Set the listing search parameters.
 *
 * @param object $params Search parameters.
 */
function gd_set_search_params( $params ) {
    global $gd_search_params;
    $gd_search_params = (array) $params;
}

/**
 * Get a listing search parameter from either memory or the query string.
 *
 * @param  string $param Parameter to get.
 * @return string        Param value.
 */
function gd_get_search_param( $param ) {
    global $gd_search_params;

    if ( isset( $gd_search_params[ $param ] ) ) {
        return esc_attr( $gd_search_params[ $param ] );
    } elseif ( isset( $_GET[ $param ] ) ) {
        return esc_attr( $_GET[ $param ] );
    } else {
        return '';
    }
}

/**
 * Checks if the agent has a source
 *
 * @param  int $source_id   ID of source.
 * @return bool           Agent has source
 */
function gd_agent_has_source( $source_id ) {
    return isset( $_SESSION['gd']['agent']->sources->{$source_id} );
}

/**
 * Checks if the user has exceeded the max listing details view count
 *
 * @param  string $listing_id   listing id of requested listing.
 * @return bool             if the user has exceeded the view count
 */
function gd_under_detail_view_limit( $listing_id ) {
    if ( isset( $_COOKIE['gd_listings_viewed'] ) ) {
        $cookie = wp_unslash( $_COOKIE['gd_listings_viewed'] );
        $gd_listings_viewed = json_decode( $cookie, true );

        // If our cookie fails to decode return false.
        if ( ! is_array( $gd_listings_viewed ) ) {
            return false;
        }
    } else {
        $gd_listings_viewed = array();
    }

    // If this is a new listing add it to the record.
    if ( ! in_array( $listing_id, $gd_listings_viewed, true ) ) {
        array_push( $gd_listings_viewed, $listing_id );
    }

    if ( count( $gd_listings_viewed ) <= $_SESSION['gd']['agent']->max_detail_views ) {
        // Expires in 10 years.
        $exp_time = time() + (10 * 365 * 24 * 60 * 60);
        setcookie( 'gd_listings_viewed', wp_json_encode( $gd_listings_viewed ), $exp_time, '/' );
        return true;
    } else {
        return false;
    }
}

/**
 * Resets the listing detail views count
 */
function gd_reset_detail_view_count() {
    if ( isset( $_COOKIE['gd_listings_viewed'] ) ) {
        unset( $_COOKIE['gd_listings_viewed'] );
        setcookie( 'gd_listings_viewed', '', time() - 3600, '/' );
    }
}

/**
 * Replaces existing query vars or adds new ones
 *
 * @param  array $vars array containing query vars to edit/add.
 * @return string new query string
 */
function gd_edit_query_vars( $vars ) {
    parse_str( $_SERVER['QUERY_STRING'], $query_vars );

    foreach ( $vars as $key => $val ) {
        $query_vars[ $key ] = $val;
    }

    return http_build_query( $query_vars );
}
/**
 * Checks if a password is at least 6 characters and contains both a number and letter
 *
 * @param  string $password password string.
 * @return bool             password is valid or not
 */
function gd_is_valid_password( $password ) {
    $is_long_enough = strlen( $password ) >= 6;
    $is_alpha_numeric = preg_match( '/[A-Za-z]/', $password ) && preg_match( '/[0-9]/', $password );
    return $is_long_enough && $is_alpha_numeric;
}

/**
 * Get agent cities for search.
 *
 * @return array Cities to allow in search.
 */
function gd_get_cities() {
    $general_options = gd_get_option( GD_OPTIONS_OUR_LISTINGS );
    if ( isset( $general_options['UseCustomCities'] ) && $general_options['UseCustomCities'] ) {
        $custom_cities = array_keys( $general_options['CustomCities'] );
        $cities        = array();

        foreach ( $custom_cities as $city ) {
            $cities[] = (object) array( 'name' => $city );
        }
    } else {
        $cities = API::call( 'GET', 'cities' );
    }

    return $cities;
}

/**
 * Process GeoDigs user signup
 *
 * @param  boolean $notify       Notify user on success.
 * @param  boolean $exit_on_fail Exit on signup failure.
 * @return array                 Errors
 */
function gd_process_signup( $notify = true, $exit_on_fail = true ) {
    $errors = array();

    if ( $_POST['email'] !== $_POST['emailConfirm'] ) {
        $errors['emails'] = 'Emails do not match';
    }
    if ( $_POST['password'] !== $_POST['passwordConfirm'] ) {
        $errors['passwords'] = 'Passwords do not match';
    }
    if ( ! gd_is_valid_password( $_POST['password'] ) ) {
        $errors['password_invalid'] = 'Password must contain at least 6 characters with at least 1 letter and 1 number';
    }
    $_POST['phone'] = preg_replace( '/\D+/', '', $_POST['phone'] );
    if ( strlen( $_POST['phone'] ) !== 10 ) {
        $errors['phone_length'] = 'Invalid phone number';
    }

    if ( count( $errors ) === 0 ) {
        $create_user = API::call( 'POST', 'users', $_POST );

        if ( isset( $create_user->error ) ) {
            if ( $exit_on_fail ) {
                wp_die( "Uh oh something went wrong signing up!  Please go back and try again or report this message to the webmaster.\nError: {$create_user->message}" );
            } else {
                $errors['process'] = $create_user->message;
            }
        } elseif ( $notify ) {
            // Send confirmation email.
            $to      = $_POST['email'];
            $subject = get_bloginfo( 'name' ) . ': Thanks for Signing Up!';
            $headers = 'From: ' . get_bloginfo( 'admin_email' ) . "\r\n" . 'Reply-To: ' . get_bloginfo( 'admin_email' ) . "\r\n" . 'X-Mailer: PHP/' . phpversion();
            $name    = $_POST['firstName']; // This is used in the email.
            $message = \GeoDigs\Template::load( 'emails/signup-confirmation.php', array( 'name' => $name ), false );
            mail( $to, $subject, $message, $headers );
        }
    }

    return $errors;
}

/**
 * Stores the current listing details to the session based on the WP query var.
 *
 * @return obj - Listing details.
 */
function gd_store_listing_details_from_query() {
    global $wp_query;

    $listing_id = $wp_query->query_vars['gd_listing_id'];
    $listing = API::call( 'GET', 'listings/' . $listing_id );
    $_SESSION['gd']['current_listing'] = $listing;

    return $listing;
}
